
rl = require('rl')

debugInfo = ''
diagnosis = '\n'
explanation = '\n'
warning = '\n'
resultString = ''

device = {}
device.diag = {} -- Registered rights functions
device.fail = {} -- List of mailfunction types
device.regRulesList = {} -- List of registered rights functions

function addExplanation(newExplanation) explanation = explanation .. '  ' .. rl.loc(newExplanation) .. '\n' end

function addWarning(newWarning) warning = warning .. ' ! ' .. rl.loc(newWarning) .. '\n' end

function addDebugInfo(newDebugInfo) 
    if device.DEBUG then 
        if newDebugInfo then
            debugInfo = debugInfo .. newDebugInfo .. '\n'
        else
            debugInfo = debugInfo .. '\n'
        end
    end 
end

function includeParentLib(fileName) dofile(libFolder .. fileName) end

-- Diagnostic functions registration
function regFunction(functionName, libInfo)
    if device.regRulesList[functionName] then
        addDebugInfo('Loaded function ' .. functionName .. ' from library ' .. libInfo .. ' function from ' .. device.regRulesList[functionName] .. ' overloaded.')
    else
        addDebugInfo('Loaded function ' .. functionName .. ' from library ' .. libInfo .. '.')
    end
    device.regRulesList[functionName] = libInfo
    return functionName
end

-- SCSI errors decoding
function ScsiSenseDecode(errCode)
    local keySense
    local codeSense
    if not errCode then return nil, 'ErrCod is nil' end
    if errCode ~= '00000000' then
        rl.includeOnce(libFolder .. 'table_fakeSense.lua')
        if fakeSense[errCode] then 
            return fakeSense[errCode]
        else
            local key, code, qualifier, FRU = string.match(errCode, '(%w%w)(%w%w)(%w%w)(%w%w)')
            if code then
                rl.includeOnce(libFolder .. 'table_scsiSense.lua')
                keySense = scsiSense[key]
                if not keySense then return nil, 'Can\'t found sense key in SCSISense table' end
            
                codeSense, errorMessage = rl.iniVal(device.BASE_DIR .. 'SCSI_Err.ini', 'SENSE', code .. qualifier)
                if not codeSense then return nil, errorMessage end
            else return nil, 'Can\'t parse SenseCode.' end
        end
    else return 'No SCSI errors.' end
    
    return keySense .. '    ' .. codeSense
end

-- GO!!! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

 -- Tags string deserialization.
for key, value in string.gmatch(QuickDiagResult,'([^\n:]+):([^\n]+)') do
    if value == value:match('%d+') then
        device[key] = tonumber(value)
    else
        device[key] = value
    end
end

addDebugInfo(QuickDiagResult)

-- dd the missing source tags
device.MFGBRAND = device.MFGBRAND or 'NOTDETECTED'
device.FAMILY = device.FAMILY or 'GNRC'
device.FAMILY_ID = device.FAMILY_ID or 0
device.FAMILY_GEN = device.FAMILY_GEN or 'GNRC'

-- End the program if there is an error in the previous stages of diagnostics
if device.STAGE_ERROR then return device.STAGE_ERROR end

-- Assigning values to auxiliary variables
libFolder = device.BASE_DIR .. 'Scripts/lib/'

-- Fill in the values of the derivate tags ======================================================================================

-- Refining the model name
do
    if device.PROFILE_TYPE == 'HDD_ATA' or device.PROFILE_TYPE == 'SSD_ATA' then 
        device.RMODEL =  string.match(device.MODEL,'%w+%s+(%w+-*%w*)')
    end

    if not device.RMODEL then device.RMODEL = device.MODEL end
end

-- Filling the code name of the product family
if (device.PROFILE_TYPE == 'HDD_ATA' or device.PROFILE_TYPE == 'SSD_ATA') and device.FAMILY_ID ~= 0 then

    if device.MFGBRAND == 'SAMSUNG' then
    
        rl.includeChunk('Scripts/lib/table_family2ID_SAMSUNG.lua')
        for family, id in pairs(family2ID) do

            if id == device.FAMILY_ID then 
                device.FAMILY = family
                break
            end
        end
        
    elseif device.MFGBRAND == 'HGST' then

        rl.includeChunk('Scripts/lib/table_family2ID_HGST.lua')
        for family, id in pairs(family2ID) do

            if id == device.FAMILY_ID then 
                device.FAMILY = family
                break
            end
        end
        
    elseif device.MFGBRAND == 'WDC' then

        device.FAMILY = 'F' .. device.FAMILY_ID
            
    elseif device.MFGBRAND == 'SEAGATE' then

        device.FAMILY = 'F' .. device.FAMILY_ID
        
    else end    
end

-- If Debug is enabled then send  additional info to output
addDebugInfo('RMODEL=>' .. device.RMODEL)
addDebugInfo('FAMILY=>' .. device.FAMILY)
addDebugInfo('FAMILY_GEN=>' .. device.FAMILY_GEN)

-- preliminary cooking of the tag contents ==================================================================================================================


-- Splitting string of SMART attributes for ATA devices-------------------------------------------------------------------------------------------------------------------------------------------
if device.PROFILE_TYPE == 'HDD_ATA' or device.PROFILE_TYPE == 'SSD_ATA' then
    local tmpTable = {}
    for key, value in pairs(device) do
        local SMART_ID = string.match(key,'^ATTR_(%d+)')
        if SMART_ID then
            tmpTable['attr' .. SMART_ID] = {}
            tmpTable['attr' .. SMART_ID].type, tmpTable['attr' .. SMART_ID].value, tmpTable['attr' .. SMART_ID].worst, tmpTable['attr' .. SMART_ID].threshold, tmpTable['attr' .. SMART_ID].raw1, tmpTable['attr' .. SMART_ID].raw2, tmpTable['attr' .. SMART_ID].raw3 = string.match(value,'^(%d+),(%d+),(%d+),(%d+),(%d+)/*(%d*)/*(%d*)$')
            for attrType, attrValue in pairs(tmpTable['attr' .. SMART_ID]) do
                if attrValue and attrValue ~= '' then
                    tmpTable['attr' .. SMART_ID][attrType] = tonumber(attrValue)
                end
            end
        end
    end
    for key, value in pairs(tmpTable) do device[key] = value end
end

-- Splitting of read test results-----------------------------------------------------------------------------------------------------------------------------------------------------------------

if device.TEST_RANDOM_SEEK then
    device.testRandomSeek = {}
    device.testRandomSeek.status, device.testRandomSeek.startLBA, device.testRandomSeek.stopLBA, device.testRandomSeek.lastLBA, device.testRandomSeek.seekCount, device.testRandomSeek.seekDone, device.testRandomSeek.errCode, device.testRandomSeek.cmdTimeMin, device.testRandomSeek.cmdTimeMax, device.testRandomSeek.cmdTimeAvg = string.match(device.TEST_RANDOM_SEEK,'^(%w+),(%d+),(%d+),(%-?%d+),(%d+),(%d+),(%w+),(%d+),(%d+),(%d+)$')
    if device.testRandomSeek.status then 
        device.testRandomSeek.startLBA = tonumber(device.testRandomSeek.startLBA); device.testRandomSeek.stopLBA = tonumber(device.testRandomSeek.stopLBA); device.testRandomSeek.lastLBA = tonumber(device.testRandomSeek.lastLBA); device.testRandomSeek.seekCount = tonumber(device.testRandomSeek.seekCount); device.testRandomSeek.seekDone = tonumber(device.testRandomSeek.seekDone); device.testRandomSeek.cmdTimeMin = tonumber(device.testRandomSeek.cmdTimeMin); device.testRandomSeek.cmdTimeMax = tonumber(device.testRandomSeek.cmdTimeMax); device.testRandomSeek.cmdTimeAvg = tonumber(device.testRandomSeek.cmdTimeAvg)
    end 
    if device.DEBUG then
        addDebugInfo('\nRandomSeek test results in expanded form:')
        for key, value in rl.pairs(device.testRandomSeek) do
            addDebugInfo('TestRandomSeek.' .. key .. ' = ' .. value)
        end
        local scsiSenseInfo, errorMessage = ScsiSenseDecode(device.testRandomSeek.errCode)
        if scsiSenseInfo then addDebugInfo('Last SCSI Sense:   ' .. scsiSenseInfo)
        else addDebugInfo('ERROR during last error code decoding:   ' .. errorMessage) end
    end
end

if device.TEST_RANDOM_READ then
    device.testRandomRead = {}
    device.testRandomRead.status, device.testRandomRead.startLBA, device.testRandomRead.stopLBA, device.testRandomRead.lastLBA, device.testRandomRead.blockSize, device.testRandomRead.readsCount, device.testRandomRead.readsDone, device.testRandomRead.errCode, device.testRandomRead.cmdTime, device.testRandomRead.testTimeMin, device.testRandomRead.testTimeMax, device.testRandomRead.testTimeAvg = string.match(device.TEST_RANDOM_READ,'^(%w+),(%d+),(%d+),(%-?%d+),(%d+),(%d+),(%d+),(%w+),(%d+),(%d+),(%d+),(%d+)$')
    if device.testRandomRead.status then 
        device.testRandomRead.startLBA = tonumber(device.testRandomRead.startLBA); device.testRandomRead.stopLBA = tonumber(device.testRandomRead.stopLBA); device.testRandomRead.lastLBA = tonumber(device.testRandomRead.lastLBA); device.testRandomRead.blockSize = tonumber(device.testRandomRead.blockSize); device.testRandomRead.readsCount = tonumber(device.testRandomRead.readsCount); device.testRandomRead.readsDone = tonumber(device.testRandomRead.readsDone); device.testRandomRead.cmdTime = tonumber(device.testRandomRead.cmdTime); device.testRandomRead.testTimeMin = tonumber(device.testRandomRead.testTimeMin); device.testRandomRead.testTimeMax = tonumber(device.testRandomRead.testTimeMax); device.testRandomRead.testTimeAvg = tonumber(device.testRandomRead.testTimeAvg)
    end
    if device.DEBUG then
        addDebugInfo('\nRandomRead test results in expanded form:')
        for key, value in rl.pairs(device.testRandomRead) do
            addDebugInfo('TestRandomRead ' .. key .. ' = ' .. value)
        end
        local scsiSenseInfo, errorMessage = ScsiSenseDecode(device.testRandomRead.errCode)
        if scsiSenseInfo then addDebugInfo('Last SCSI Sense:   ' .. scsiSenseInfo)
        else addDebugInfo('ERROR during last error code decoding:   ' .. errorMessage) end
    end
end

function dddSplit(testName, tagVal)
    if tagVal then
        local table = {}
        table.status, table.startLBA, table.stopLBA, table.blockSize, table.errCount, table.errCode, table.testTime = string.match(tagVal,'^(%w+),(%d+),(%d+),(%d+),(%d+),(%w+),(%d+)$')
        if table.status then 
            table.startLBA = tonumber(table.startLBA); table.stopLBA = tonumber(table.stopLBA); table.blockSize = tonumber(table.blockSize); table.errCount = tonumber(table.errCount); table.testTime = tonumber(table.testTime)
        end
        if device.DEBUG then
            addDebugInfo('\n' .. testName ..' test results in expanded form:')
            for key, value in rl.pairs(table) do
                addDebugInfo(testName .. ' ' .. key .. ' = ' .. value)
            end
            local scsiSenseInfo, errorMessage = ScsiSenseDecode(table.errCode)
            if scsiSenseInfo then addDebugInfo('Last SCSI Sense:   ' .. scsiSenseInfo)
            else addDebugInfo('ERROR during last error code decoding:   ' .. errorMessage) end
        end
    return table
    end
end 

device.testReadOD = dddSplit('TestReadOD', device.TEST_READ_OD)
device.testReadMD = dddSplit('TestReadMD', device.TEST_READ_MD)
device.testReadID = dddSplit('TestReadID', device.TEST_READ_ID)

addDebugInfo()


-- Diagnosis on the basis of the information provided====================================================================================================

--  Search and load an appropriate library---------------------------------------------------------------------------------------------------------------------------
do
    function libExists(libName)
        local fileName = libFolder .. libName .. '.lua'
        if rl.fileExists(fileName) then return fileName else return false end
    end

    libFileName = libExists(device.PROFILE_TYPE .. '_' .. device.MFGBRAND  .. '_' .. device.RMODEL .. '_' .. device.FWVER) 
                or libExists(device.PROFILE_TYPE .. '_' .. device.MFGBRAND  .. '_' .. device.RMODEL) 
--              or libExists(device.PROFILE_TYPE .. '_' .. device.MFGBRAND  .. '_' .. device.FAMILY) 
                or libExists(device.PROFILE_TYPE .. '_' .. device.MFGBRAND) 
                or libExists(device.PROFILE_TYPE)
                
    if not libFileName then 
        errorMessage = 'Can\'t found any lib file for this device.\n\n'
        return errorMessage .. '\n\n' .. resultString .. debugInfo
    else 
        if device.DEBUG then resultString = 'Loaded library file ' .. libFileName .. '\n' .. resultString  end
    end
end

local success, errorMessage = rl.protectedLoad(libFileName)

-- Perform diagnostic functions ---------------------------------------------------------------------------------------------------------------------------

if success then
    addDebugInfo()
    for diagRuleName in pairs(device.diag) do 
	globalVariableForNameDiagRuleApplied = diagRuleName
        success, errorMessage = pcall(device.diag[diagRuleName])
        if success then
            addDebugInfo('Executed function ' .. diagRuleName .. ' from library '.. device.regRulesList[diagRuleName] .. '.')
        else 
            return 'Error in the function ' ..  diagRuleName .. ' from library '.. device.regRulesList[diagRuleName] .. ':\n' .. errorMessage .. '\n\n' .. resultString .. debugInfo
        end
    end
else
    return errorMessage .. '\n\n' .. resultString .. debugInfo
end

-- Calling methods, which describe malfunctions ---------------------------------------------------------------------------------------------------------------------------

local malfunctionDetected = false
for diagResultName in pairs(device.fail) do
    local verdict = device.fail[diagResultName]:verdict()
    if verdict then diagnosis = diagnosis .. verdict end
    if not malfunctionDetected then 
        malfunctionDetected = device.fail[diagResultName]:malfunctionDetected() 
    end 
end

-- Formatting output---------------------------------------------------------------------------------------------------------------------------------------------

local intro
local finally



if not device.DEBUG then
    if  device.NOTDETECTED then 
        if malfunctionDetected then
            intro = rl.loc('Based on the data provided, we can assume the following kind of the malfunction:')
            finally = rl.loc('')
            if string.find(device.PROFILE_TYPE, 'HDD') then         
                finally = finally .. rl.loc('To clarify the diagnosis and data recovery, we recommend you contact with experienced specialists which have special equipment, such as PC-3000.')
            end
        else
            intro = rl.loc('Not enough data to determine the kind of the malfunction.')
            if string.find(device.PROFILE_TYPE, 'HDD') then         
                finally = rl.loc('The sounds from hard drive is main information to determine the kind of mulfuctions. Please pay special attention to this item of the questionnaire.')
            end
        end     
    else
        if malfunctionDetected then
            intro = rl.loc('Identified signs of hardware failure:')
            finally = rl.loc('To clarify the diagnosis and data recovery, we recommend you contact with experienced specialists which have special equipment, such as PC-3000.')
        else
            if string.find(device.PROFILE_TYPE, 'HDD') and device.FORM_DATA_IMPORTANCE == 'CRITICAL' and device.FORM_MECHANICAL_SHOCK == 'YES' then
                intro = rl.loc('If there are suspicions of mechanical damage of drive which have a critical data, diagnostics are not performed.')
                finally = rl.loc('In this case need to chek heads conditions to prevent self distructiun of hard drive. Only experienced specialists can do it.')
            else
                intro = rl.loc('No signs of hardware faults detected.')
                finally = rl.loc('If data is not available, this may be due to file system corruption. You can try using a data recovery program, such as UFS Explorer.')
            end
        end     
    end
    finally = finally .. '\n\n' .. rl.loc('disclaimer')
end

if intro and intro ~= '\n' then resultString = resultString .. intro .. '\n' end
if diagnosis and diagnosis ~= '\n' then resultString = resultString .. diagnosis end
if explanation and explanation ~= '\n' then resultString = resultString .. '\n' .. rl.loc('<===< EXPLANATION >===>') .. explanation end
if warning and warning ~= '\n' then resultString = resultString .. '\n' ..  rl.loc('!!! WARNING !!!') .. warning end
if finally and finally ~= '\n' then resultString = resultString  .. '\n' .. finally .. '\n' end




if device.DBG_CONTAINER then resultString = '+++CONTAINER VIEWING MODE+++\n' .. resultString .. '\n###STORED INFORMATION###\n' .. QuickDiagResult end
if device.DEBUG then resultString = '***DEBUG MODE***\n' .. resultString .. '\n###DEBUG INFO###\n' .. debugInfo end

return resultString

